# This file is read by Autoconf.                       -*- Autoconf -*-
#
#   Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
#   2003, The Regents of the University of California, through Lawrence
#   Berkeley National Laboratory (subject to receipt of any required
#   approvals from the U.S. Dept. of Energy).  All rights reserved.
#
#   Portions may be copyrighted by others, as may be noted in specific
#   copyright notices within specific files.
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: acinclude.m4,v 1.41 2004/10/18 20:08:14 phargrov Exp $
AC_REVISION($Revision: 1.41 $)

# CR_IF(test,[ACTION-IF-FOUND],[ACTION-IF-FOUND])
# ------------------------------------------------------
# Basic functionality of AS_IF, but not as "optimized"
# Implemented here because m4sh is not yet documented.
AC_DEFUN([CR_IF],[
  if $1; then
    m4_default([$2], :)
  else
    m4_default([$3], :)
  fi
])

# CR_PROG_GCC([LIST],[ACTION-IF-FOUND],[ACTION-IF-NOT-FOUND])
# ------------------------------------------------------
# Find the first compiler in the list and verify that it is gcc.
AC_DEFUN([CR_PROG_GCC],[
  AC_PROG_CC($1)
  CR_IF([test "$GCC" = yes],[$2],[$3])
])

# CR_CHECK_GLIBC(MAJOR,MINOR,[ACTION-IF-FOUND],[ACTION-IF-NOT-FOUND])
# ------------------------------------------------------
# Check for glibc >= MAJOR.MINOR and sets the variable
# cr_cv_check_glibc_MAJOR_MINOR_or_higher to 'yes' or 'no'
AC_DEFUN([CR_CHECK_GLIBC],[
  cr_cvname="cr_cv_check_glibc_"$1"_"$2"_or_higher"
  AC_CACHE_CHECK([[for GNU libc version >= $1.$2]],[$cr_cvname],[
    AC_TRY_RUN([
	#include <features.h>
	#include <stdlib.h>
	#ifndef __GLIBC_PREREQ
	    #define __GLIBC_PREREQ(maj, min) \
		((__GLIBC__ << 16) + __GLIBC_MINOR__ >= ((maj) << 16) + (min))
	#endif
	int main ()
	{
	    exit((gnu_get_libc_version() && __GLIBC_PREREQ($1, $2)) ? 0 : 1);
	}
    ], [eval $cr_cvname=yes], [eval $cr_cvname=no])
  ])
  CR_IF([eval test \$$cr_cvname = yes],[$3],[$4])
])

# CR_KERNEL_TYPE(TYPE)
# ------------------------------------------------------
# Check for /boot/kernel.h and establish user-requested overrides
# if possible.
AC_DEFUN([CR_KERNEL_TYPE],[
  cr_kernel_type=`echo $1 | tr 'a-z' 'A-Z'`
  cr_header="/boot/kernel.h"
  AC_MSG_CHECKING([for $cr_header]);
  cr_result=ok
  if test \! -e "$cr_header"; then
    cr_result='missing'
  elif test \! -r "$cr_header"; then
    cr_result='not readable'
  fi
  AC_MSG_RESULT([$cr_result])
  if test "$cr_result" != "ok"; then
    AC_MSG_ERROR([You have requested '--with-kernel-type=$cr_kernel_type', but $cr_header is $cr_result.])
  fi
  cr_kernel_var="__BOOT_KERNEL_$cr_kernel_type"
  if test -z "`grep \"$cr_kernel_var\" $cr_header 2>/dev/null`"; then
    AC_MSG_ERROR([You have requested '--with-kernel-type=$cr_kernel_type', but $cr_header does not appear to support that type.])
  fi
  # The following sed command transforms all the #ifndef lines from kernel.h into
  # corresponding preprocessor flags which select the desired kernel type.
  # The tr command flattens the multiline output (because the backtics don't always do so).
  CR_KTYPE_CPPFLAGS=`sed -n -e 's/^#ifndef \('$cr_kernel_var'\)$/-D\1=1 /p' \
			    -e 's/^#ifndef \(__BOOT_KERNEL_[A-Z]*\)$/-D\1=0 /p' \
			    $cr_header \
		     | tr -s ' \n\r' ' '`
])

# CR_CHECK_LINUX(MAJOR,MINOR,[ACTION-IF-FOUND],[ACTION-IF-NOT-FOUND])
# ------------------------------------------------------
# Check for source (headers actually) of the specified MAJOR.MINOR
# version of Linux, and set LINUX_SRC accordingly.
# If LINUX_SRC is already set, just verify it w/o searching.
# Verification consists of ensuring that linux/version.h exists
# and has the correct MAJOR and MINOR version.
# Any patch-level checking should be done elsewhere.
AC_DEFUN([_CR_CHECK_LINUX],[
  SAVE_CPPFLAGS=$CPPFLAGS
  CPPFLAGS="-I$3/include -D__KERNEL__ -DMODULE $CR_KTYPE_CPPFLAGS $CPPFLAGS"
  AC_TRY_RUN([
	#include <stdio.h>
	#include "$3/include/linux/version.h"
	int main ()
	{
		FILE *f=fopen("conftestval", "w");
  		if (!f) return(1);

  		fputs(UTS_RELEASE, f); fputc('\n', f);

		return !((LINUX_VERSION_CODE >= KERNEL_VERSION($1,$2,0)) &&
			 (LINUX_VERSION_CODE <  KERNEL_VERSION($1,$2+1,0)));
	}
  ], [cr_linux_dir_ok=yes;
      cr_linux_ver=`cat conftestval`],
     [cr_linux_dir_ok=no])
  CPPFLAGS=$SAVE_CPPFLAGS
])
AC_DEFUN([CR_CHECK_LINUX],[
  AC_SUBST([LINUX_SRC])
  AC_SUBST([LINUX_VER])
  AC_SUBST([CR_KERNEL])
  AC_SUBST([CR_KERNEL_BASE])
  AC_SUBST([CR_MODULE_DIR])
  AC_MSG_CHECKING([[for Linux kernel $1.$2 headers]])
  if test -n "$LINUX_SRC" ; then
    # Just validate the user's setting
    _CR_CHECK_LINUX([$1],[$2],[$LINUX_SRC])
  else
    # Search standard locations
    for cr_linux_dir in /usr/src/linux-`uname -r` \
			/lib/modules/`uname -r`/build \
			/usr/src/linux-$1.$2 \
			/usr/src/linux; do
      _CR_CHECK_LINUX([$1],[$2],[$cr_linux_dir])
      if test $cr_linux_dir_ok = yes; then
	LINUX_SRC=$cr_linux_dir
	break
      fi
   done
  fi
  CR_IF([test $cr_linux_dir_ok = yes],
	[LINUX_VER=$cr_linux_ver
	 CR_KERNEL=`echo $cr_linux_ver | tr - _`
	 CR_KERNEL_BASE=`echo $CR_KERNEL | sed -e 's:smp\|enterprise\|bigmem::g'`
	 CR_MODULE_DIR="${libdir}/${PACKAGE}/${LINUX_VER}"
	 AC_MSG_RESULT([$LINUX_SRC (version $LINUX_VER)])
         $3],
	[AC_MSG_RESULT([not found${LINUX_SRC:+ in $LINUX_SRC}]); $4])
])

# CR_SET_KCFLAGS()
# ------------------------------------------------------
# Sets the architecture dependent CR_K*FLAGS SUBSTs
AC_DEFUN([CR_SET_KCFLAGS],[
  AC_MSG_CHECKING([[for flags to compile Linux kernel modules]])
  AC_ARG_VAR([KCFLAGS], [prepended to CFLAGS when compiling kernel modules])
  AC_ARG_VAR([KCPPFLAGS], [prepended to CPPFLAGS when compiling kernel modules])
  AC_ARG_VAR([KLDFLAGS], [appended to LDFLAGS when linking kernel modules])
  AC_REQUIRE([AC_CANONICAL_HOST])
  AC_REQUIRE([CR_CHECK_LINUX])
  cr_kcppflags="-I$LINUX_SRC/include -D__KERNEL__ -DMODULE $CR_KTYPE_CPPFLAGS"
  cr_kcflags="-Wall -Wstrict-prototypes -O2 -fomit-frame-pointer"
  case $host_cpu in
    i?86)
	cr_kcflags="$cr_kcflags -fno-strength-reduce"
	;;
    sparc)
	cr_kcflags="$cr_kcflags -m32 -mno-fpu -fcall-used-g5 -fcall-used-g7"
	;;
    alpha)
	cr_kcflags="$cr_kcflags -mno-fp-regs -ffixed-8 -mcpu=ev5"
	;;
    ppc)
	cr_kcppflags="$cr_kcppflags -D__powerpc__"
	cr_kcflags="$cr_kcflags -fsigned-char -msoft-float -pipe \
		    -ffixed-r2 -Wno-uninitialized -mmultiple -mstring"
	;;
    *)
	AC_MSG_WARN([Unknown host cpu "$host_cpu", using defaults])
	;;
  esac
  AC_SUBST([CR_KCFLAGS],[$cr_kcflags])
  AC_SUBST([CR_KCPPFLAGS],[$cr_kcppflags])
  AC_SUBST([CR_KLDFLAGS],['-S -X -r'])
  AC_MSG_RESULT([ok])
])

# CR_DEFINE(VARIABLE,TEST,[TEMPLATE_TEXT])
# ------------------------------------------------------
# Wrapper to invoke AC_DEFINE for a boolean, and optionally AH_TEMPLATE
AC_DEFUN([CR_DEFINE],[
  if [$2]; then
    AC_DEFINE([$1], [1])
  else
    AC_DEFINE([$1], [0])
  fi
  m4_ifval([$3], AH_TEMPLATE([$1], [$3]))
])

# CR_DEFINE_AND_SET(VARIABLE,TEST,[TEMPLATE_TEXT])
# ------------------------------------------------------
# Wrapper like CR_DEFINE which also sets the corresponding shell variable
# to either 1 or the empty string
AC_DEFUN([CR_DEFINE_AND_SET],[
  if [$2]; then
    AC_DEFINE([$1], [1])
    eval "$1=1"
  else
    AC_DEFINE([$1], [0])
    eval "$1=''"
  fi
  m4_ifval([$3], AH_TEMPLATE([$1], [$3]))
])

# CR_ARG_ENABLE(NAME,HELP,[DEFAULT])
# ------------------------------------------------------
# Our own wrapper for AC_ARG_ENABLE
# Default for DEFAULT is "no"
AC_DEFUN([CR_ARG_ENABLE],[
  AC_ARG_ENABLE([$1],
    [AC_HELP_STRING([--enable-$1], [$2])],
    :,
    [[enable_]patsubst([$1], -, _)=m4_default([$3], [no])])
])

# CR_CHECK_SIGNUM
# ------------------------------------------------------
# Probes for value to assign to CR_SIGNUM
AC_DEFUN([CR_CHECK_SIGNUM],[
AC_MSG_CHECKING(for value of CR_SIGNUM)
cr_signum="failed"
SAVE_LIBS="$LIBS"
LIBS="-ldl -lpthread $LIBS"
AC_TRY_RUN([#include <stdio.h>
extern int __libc_allocate_rtsig(int);
int main()
{
  FILE *f=fopen("conftestval", "w");
  int s=__libc_allocate_rtsig(0);
  if (!f || s<=0) return(1);
  fprintf(f, "%d\n", s);
  return(0);
}], cr_signum=`cat conftestval`, AC_MSG_ERROR([Failed to probe CR_SIGNUM]), AC_MSG_ERROR([Cannot probe CR_SIGNUM when cross compiling]))
AC_SUBST([CR_SIGNUM],[$cr_signum])
LIBS="$SAVE_LIBS"
AC_MSG_RESULT([$cr_signum])
])

# CR_CONFIG_REPORT
# ------------------------------------------------------
# Reports certain probed/configured values
AC_DEFUN([CR_CONFIG_REPORT],[
  echo "======================================================================"
  echo "Please review the following configuration information:"
  echo "  Kernel source directory = $LINUX_SRC"
  echo "  Kernel symbol table = ${LINUX_SYSTEM_MAP}${LINUX_VMLINUX}"
  echo "  Kernel version probed from source = $LINUX_VER"
  echo "  Kernel running currently = `uname -r`"
  echo "======================================================================"
])

# CR_TRY_KERNEL_COMPILE(INCLUDES,PROGRAM,[ACTION-IF-TRUE],[ACTION-IF_FALSE])
# ------------------------------------------------------
# Wrapper to invoke AC_TRY_COMPILE against the kernel headers
AC_DEFUN([CR_TRY_KERNEL_COMPILE],[
  AC_REQUIRE([CR_SET_KCFLAGS])
  SAVE_CFLAGS=$CFLAGS
  SAVE_CPPFLAGS=$CPPFLAGS
  CFLAGS="$CR_KCFLAGS"
  CPPFLAGS="$CR_KCPPFLAGS"
  # Note the patsubst to ensure we never mistakenly pick up a file from /usr/include/linux.
  # We can't simply use -nostdinc, because the kernel needs stdarg.h and maybe others.
  AC_TRY_COMPILE([#include <${LINUX_SRC}/include/linux/kernel.h>]
		 patsubst([$1], [<\(\w+/\)], [<${LINUX_SRC}/include/\1]),
		 [$2], [cr_result="yes"],[cr_result="no"])
  CFLAGS=$SAVE_CFLAGS
  CPPFLAGS=$SAVE_CPPFLAGS
  CR_IF([test x"$cr_result" = "xyes"], [$3], [$4])
])

# CR_CHECK_KERNEL_COMPILE(SYMBOL,INCLUDES,PROGRAM,[TEMPLATE_TEXT])
# ------------------------------------------------------
# Wrapper to invoke CR_TRY_KERNEL_COMPILE then AC_DEFINE.
# Calls AH_TEMPLATE with either default or supplied text.
# Also sets HAVE_... to 1 or empty string
AC_DEFUN([CR_CHECK_KERNEL_COMPILE],[
  AC_MSG_CHECKING([kernel for $1])
  CR_TRY_KERNEL_COMPILE([$2],[$3],[cr_result=yes],[cr_result=no])
  CR_DEFINE_AND_SET(AS_TR_CPP(HAVE_[]$1),[test x"$cr_result" = xyes],
    m4_default([$4], [[Define to 1 if the kernel has $1]]))
  AC_MSG_RESULT([$cr_result])
])

# CR_BAD_KERNEL([REASON])
# ------------------------------------------------------
# Die, informing the user that the kernel cannot be used.
# Optionally provide a reason.
AC_DEFUN([CR_BAD_KERNEL],[
  AC_CACHE_SAVE
  CR_CONFIG_REPORT()
  m4_ifval([$1],
   [ AC_MSG_ERROR([Unable to use kernel $LINUX_VER - $1]) ],
   [ AC_MSG_ERROR([Unable to use kernel $LINUX_VER]) ])
])

# CR_LINUX_SYMTAB()
# ------------------------------------------------------
# Check for System.map in some standard places
# If LINUX_SYSTEM_MAP is set, just verifies existance,
# else if LINUX_VMLINUX is set, just verifies existance,
# else search for either file in standard locations.
# When complete sets LINUX_SYMTAB_CMD such that
# "eval $LINUX_SYMTAB_CMD" will produce a System.map on stdout.
#
# XXX should do some sanity checks on the files supplied by
# the user to see thet have the expected format(s).
AC_DEFUN([_CR_CHECK_SYSTEM_MAP],[
  if test -n "$1" -a -f "$1" && grep ' A _text' <"$1" >/dev/null 2>/dev/null; then
    LINUX_SYSTEM_MAP="$1"
    LINUX_SYMTAB_FILE="$1"
    LINUX_SYMTAB_CMD="cat $1"
  fi
])
AC_DEFUN([_CR_CHECK_VMLINUX],[
  AC_REQUIRE([AC_PROG_NM])
  if test -n "$1" -a -f "$1" & ($NM "$1" | grep ' A _text') >/dev/null 2>/dev/null; then
    LINUX_VMLINUX="$1"
    LINUX_SYMTAB_FILE="$1"
    LINUX_SYMTAB_CMD="$NM $1"
  fi
])
AC_DEFUN([CR_LINUX_SYMTAB],[  
  AC_REQUIRE([CR_CHECK_LINUX])
  AC_MSG_CHECKING([[for Linux kernel symbol table]])
  AC_SUBST([LINUX_SYMTAB_FILE])
  AC_SUBST([LINUX_SYMTAB_CMD])
  AC_SUBST([LINUX_SYMTAB_CONF])
  LINUX_SYMTAB_CMD=""
  # First try validating the user's selection
  if test -n "$LINUX_SYSTEM_MAP" ; then
    _CR_CHECK_SYSTEM_MAP(["$LINUX_SYSTEM_MAP"])
    if test -z "$LINUX_SYMTAB_CMD"; then
      # The user specified a file, but we can't use it.  Abort.
      AC_MSG_RESULT([failed])
      AC_MSG_ERROR([Failed to validate "$LINUX_SYSTEM_MAP"])
    else
      # Save use in .spec file
      LINUX_SYMTAB_CONF="--with-system-map=$1"
    fi
  fi
  if test -z "$LINUX_SYMTAB_CMD" -a -n "$LINUX_VMLINUX" ; then
    _CR_CHECK_VMLINUX(["$LINUX_VMLINUX"])
    if test -z "$LINUX_SYMTAB_CMD"; then
      # The user specified a file, but we can't use it.  Abort.
      AC_MSG_RESULT([failed])
      AC_MSG_ERROR([Failed to validate "$LINUX_VMLINUX"])
    else
      # Save use in .spec file
      LINUX_SYMTAB_CONF="--with-vmlinux=$1"
    fi
  fi
  # Next try searching for System.map or vmlinux in standard locations
  # Note we use the kernel version found in the headers, not `uname -r`
  if test -z "$LINUX_SYMTAB_CMD" ; then
    for cr_file_pattern in "$LINUX_SRC/@-$LINUX_VER" \
			   "/boot/@-$LINUX_VER"      \
			   "/@-$LINUX_VER"           \
			   "$LINUX_SRC/@" ; do
      # Try System.map in the given location
      cr_file=`echo $cr_file_pattern | sed -e 's|@|System.map|'`
      _CR_CHECK_SYSTEM_MAP([$cr_file])
      if test -n "$LINUX_SYMTAB_CMD"; then
	break
      fi
      # Try vmlinux in the given location
      cr_file=`echo $cr_file_pattern | sed -e 's|@|vmlinux|'`
      _CR_CHECK_VMLINUX([$cr_file])
      if test -n "$LINUX_SYMTAB_CMD"; then
	break
      fi
    done
  fi
  # Announce our result
  if test -z "$LINUX_SYMTAB_CMD"; then
    LINUX_SYMTAB_CMD="true"
    AC_MSG_RESULT([failed])
    AC_MSG_ERROR([Failed to locate kernel symbol table.  Try using --with-system-map or --with-vmlinux.])
  else
    AC_MSG_RESULT([$LINUX_SYMTAB_FILE])
  fi
  # Now check for consistency w/ the kernel source
  # XXX: Currently just check SMPness. Can this be more aggressive?
  AC_MSG_CHECKING([for SMP kernel source])
  CR_TRY_KERNEL_COMPILE([
	#include <linux/config.h>
	#ifndef CONFIG_SMP
	    choke me
	#endif
	],[],[cr_kernel_smp=yes],[cr_kernel_smp=no])
  AC_MSG_RESULT([$cr_kernel_smp]);
  AC_MSG_CHECKING([for SMP kernel symbol table])
  cr_symtab_smp=no
  if test -n "`eval $LINUX_SYMTAB_CMD | grep del_timer_sync 2>/dev/null`"; then
    cr_symtab_smp=yes
  fi
  AC_MSG_RESULT([$cr_symtab_smp]);
  if test "$cr_kernel_smp" != "$cr_symtab_smp"; then
    CR_CONFIG_REPORT()
    if test "$cr_kernel_smp" = yes; then
      AC_MSG_ERROR([Kernel source is configured SMP but the kernel symbol table is not.  Consider specifying a symbol table with --with-system-map or --with-vmlinux.  Or, if using kernel sources that are configured by /boot/kernel.h, you may try --with-kernel-type=UP to force a uni-processor interpretation of the sources.])
    else
      AC_MSG_ERROR([Kernel source is configured uni-processor but the kernel symbol table is SMP.  Consider specifying a symbol table with --with-system-map or --with-vmlinux.  Or, if using kernel sources that are configured by /boot/kernel.h, you may try --with-kernel-type=SMP to force an SMP interpretation of the sources.])
    fi
  fi
])

# CR_FIND_KSYM(SYMBOL,[UNCONDITIONAL])
# ------------------------------------------------------
# Search System.map for address of the given symbol and
# do one of three things in config.h:
# If not found, leave CR_KSYM_foo undefined
# If found to be exported, "#define CR_KSYM_foo 0"
# If found not to be exported, "#define CR_KSYM_foo 0x<value>"
# If the optional UNCONDITIONAL argument is provided 
# then the value is saved to config.h even if exported.
AC_DEFUN([CR_FIND_KSYM],[
  AC_REQUIRE([CR_LINUX_SYMTAB])
  AC_MSG_CHECKING([[kernel symbol table for $1]])
  cr_addr=`eval $LINUX_SYMTAB_CMD | grep " $1\$" | cut -d\  -f1`
  if test -z $cr_addr; then
    AC_MSG_RESULT([not found])
  else
    cr_result=$cr_addr
    cr_addr="0x$cr_addr"
    m4_ifval([$2],[],[
      if `eval $LINUX_SYMTAB_CMD | grep " __ksymtab_$1\$" >/dev/null`; then
        cr_result="exported"
        cr_addr="0"
      fi
    ])
    AC_MSG_RESULT([$cr_result])
    AC_DEFINE_UNQUOTED(CR_KSYM_[]$1,$cr_addr,
			[Define to address of kernel symbol $1, or 0 if exported])
  fi
])

# Find how to rpmbuild (if possible at all)
AC_DEFUN([CR_PROG_RPMBUILD],[
  AC_PATH_PROGS([RPMBUILD],[rpmbuild rpmb rpm],[none],[$PATH:/usr/lib/rpm])
  if $RPMBUILD -bs 2>&1 | grep 'no spec' >/dev/null 2>/dev/null; then
    :
  else
    RPMBUILD=none
  fi
  AM_CONDITIONAL(HAVE_RPMBUILD,[test $RPMBUILD != none])
])

# CR_CONFIG_SSS()
# ------------------------------------------------------
# Configure Scalable Systems Software stuff
AC_DEFUN([CR_CONFIG_SSS],[
  AC_MSG_CHECKING([for MPD_HOME])
  AC_ARG_WITH([mpd],
    [AC_HELP_STRING([--with-mpd=DIR],[specify path to the mpd installation directory (default is to search using PATH)])],
    [AS_IF([test $with_mpd != yes],[MPD_HOME=$with_mpd])])
  if test -n "$MPD_HOME"; then
    AC_MSG_RESULT([$MPD_HOME])
  else
    AC_MSG_RESULT([not set])
    AC_PATH_PROG([MPDCMD],[mpd],[not found],[${PATH}:/usr/mpi/bin])
    if test "$MPDCMD" = "not found"; then
      AC_MSG_ERROR([--enable-sss requires path to the mpd instalation directory which could not be found in the default location, or by using PATH.  Please use --with-mpd=dir to specify the mpd installation directory.])
    else
      MPD_HOME=`dirname $MPDCMD`
      MPD_HOME=`dirname $MPD_HOME`
    fi
  fi
  SAVE_CPPFLAGS=$CPPFLAGS
  SAVE_LDFLAGS=$LDFLAGS
  CPPFLAGS="$CPPFLAGS -I${MPD_HOME}/include"
  LDFLAGS="$LDFLAGS -L${MPD_HOME}/lib"
  # First just check for availability
  AC_CHECK_LIB([mpich],[PMI_Init],[:],[AC_MSG_ERROR([Unable to link to PMI library required by --enable-sss])])
  AC_CHECK_HEADERS([pmi.h])
  # Next check for sufficiently recent versions
  AC_CHECK_LIB([mpich],[PMI_Get_id],[:],[AC_MSG_ERROR([${MPD_HOME}/lib/libmpich.a contains an older unsupported PMI version.])])
  if test "$ac_cv_header_pmi_h" = "yes"; then
    AC_MSG_CHECKING([for pmi.h sufficiently recent])
    AC_TRY_COMPILE([#include <pmi.h>],[
		    int err, is_init;
		    err = PMI_Initialized(&is_init);
		   ], [cr_result="yes"],[cr_result="no"])
    AC_MSG_RESULT($cr_result)
    if test "$cr_result" != "yes"; then
      AC_MSG_ERROR([${MPD_HOME}/pmi.h is from an older unsupported PMI version.])
    fi
  fi
  CPPFLAGS=$SAVE_CPPFLAGS
  LDFLAGS=$SAVE_LDFLAGS
  AC_SUBST(MPD_HOME)
  AC_DEFINE_UNQUOTED([MPD_HOME], "$MPD_HOME")
  AH_TEMPLATE([MPD_HOME],[MPD installation directory (if needed)])

  AC_MSG_CHECKING([for LAM_HOME])
  AC_ARG_WITH([lam],
    [AC_HELP_STRING([--with-lam=DIR],[specify path to LAM/MPI installation directory (default is to search using PATH)])],
    [AS_IF([test $with_lam != yes],[LAM_HOME=$with_lam])])
  # Note that we must not try to verify LAM_HOME, since in a normal build
  # sequence we must be installed before the corresponding LAM can be built.
  if test -n "$LAM_HOME"; then
    AC_MSG_RESULT([$LAM_HOME])
  else
    AC_MSG_RESULT([not set])
    AC_PATH_PROG([LAMCMD],[lamboot],[not found])
    if test "$LAMCMD" = "not found"; then
      AC_MSG_ERROR([--enable-sss requires the path to the LAM/MPI installation directory which could not be found using PATH.  Please use --with-lam=dir to specify the lam/mpi installation directory.])
    else
      LAM_HOME=`dirname $LAMCMD`
      LAM_HOME=`dirname $LAM_HOME`
    fi
  fi
  AC_SUBST(LAM_HOME)
  AC_DEFINE_UNQUOTED([LAM_HOME], "$LAM_HOME")
  AH_TEMPLATE([LAM_HOME],[LAM/MPI installation directory (if needed)])
])
