/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: get_info.c,v 1.1 2004/01/22 17:12:55 phargrov Exp $
 */

#include <sys/types.h>
#include <unistd.h>
#include <limits.h>
#include "libcr.h"

#define ASSERT(COND) if (!(COND)) exit(1)

char dest[PATH_MAX];

int result = 1;	/* assume failure */

static int my_callback(void* arg)
{
    const struct cr_checkpoint_info *info = cr_get_checkpoint_info();

    result = !info || strcmp(info->dest, dest) || info->signal;

    return 0;
}

int main(void)
{
    char *p;
    int len;
    int rc;

    p = getcwd(dest, PATH_MAX);
    ASSERT(p == dest);

    len = strlen(dest);
    ASSERT(len > 0);

    rc = snprintf(dest+len, PATH_MAX-len, "/context.%d", (int)getpid());
    ASSERT((rc > 0) && ((rc + len) < PATH_MAX));

    rc = (int)cr_init();
    ASSERT(rc >= 0);

    rc = (int)cr_register_callback(my_callback, NULL, CR_SIGNAL_CONTEXT);
    ASSERT(rc >= 0);

    cr_request();

    rc = unlink(dest);
    ASSERT(rc >= 0);

    return result;
}
